#define G_LOG_DOMAIN "gpod-feed-widget"

#include "config.h"

#include "gpod-feed-widget.h"

#include <gtk/gtk.h>
#include <glib/gi18n.h>

/**
 * SECTION:gpod-feed-widget
 * @title: GpodFeedWidget
 * @short_description: A #GtkBox widget for a feed to be used in #GpodFeedView
 * @include: gpod-feed-widget.h
 * @see_also: #GpodFeed, #GpodFeedView
 *
 * The #GpodFeedWidget is an instance #GtkBox, and acts as the primary widget
 * inside of #GpodFeedView.
 **/

typedef struct _GpodFeedWidgetPrivate
{
    gchar *title_text;
    gchar *image_path;

    GtkWidget *box;
    GtkWidget *title;
    GtkWidget *image;
} GpodFeedWidgetPrivate;

struct _GpodFeedWidget
{
    GtkFlowBoxChild parent;
};

G_DEFINE_TYPE_WITH_PRIVATE (GpodFeedWidget, gpod_feed_widget, GTK_TYPE_FLOW_BOX_CHILD)

enum
{
    PROP_0,
    PROP_TITLE,
    PROP_IMAGE,
    NUM_PROPERTIES
};

static GParamSpec *properties[NUM_PROPERTIES] = { NULL, };

static void
gpod_feed_widget_set_property (GObject      *object,
                               guint         property_id,
                               const GValue *value,
                               GParamSpec   *pspec)
{
    GpodFeedWidget *self = GPOD_FEED_WIDGET (object);
    GpodFeedWidgetPrivate *priv = gpod_feed_widget_get_instance_private (self);

    switch (property_id)
    {
        case PROP_TITLE:
            priv->title_text = g_value_dup_string (value);
            break;

        case PROP_IMAGE:
            priv->image_path = g_value_dup_string (value);
            break;

        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
            break;
    }
}

static void
gpod_feed_widget_get_property (GObject      *object,
                               guint         property_id,
                               GValue       *value,
                               GParamSpec   *pspec)
{
    GpodFeedWidget *self = GPOD_FEED_WIDGET (object);
    GpodFeedWidgetPrivate *priv = gpod_feed_widget_get_instance_private (self);

    switch (property_id)
    {
        case PROP_TITLE:
            g_value_set_string (value, priv->title_text);
            break;

        case PROP_IMAGE:
            g_value_set_string (value, priv->image_path);
            break;

        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
            break;
    }
}

/*
 * gpod_feed_widget_finalize:
 * @object: An instance of #GpodFeedWidget
 *
 * Does any last minute clean-ups when the class ends.
 */
static void
gpod_feed_widget_finalize (GObject *object)
{
    G_OBJECT_CLASS (gpod_feed_widget_parent_class)->finalize(object);
}

/*
 * gpod_feed_widget_init:
 * @self: An instance of #GpodFeedWidget
 *
 * Initializes various aspects of the #GpodFeedWidget. Anything the #GpodFeedWidget
 * would need before becoming presented should be done here.
 */
static void
gpod_feed_widget_init (GpodFeedWidget *self)
{
    /* Init widgets/feed.ui template */
    gtk_widget_init_template (GTK_WIDGET (self));
}

/*
 * gpod_feed_widget_class_init:
 * @klass: An instance of #GpodFeedWidgetClass
 *
 * Initializes the various virtural functions in #GpodFeedWidgetClass. In
 * addtition it connects the various #GtkWidgets from the gresource file using
 * gtk_widget_class_set_template_from_resource().
 */
static void
gpod_feed_widget_class_init (GpodFeedWidgetClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);
    GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);

    object_class->set_property = gpod_feed_widget_set_property;
    object_class->get_property = gpod_feed_widget_get_property;
    object_class->finalize = gpod_feed_widget_finalize;

	/**
	 * GpodFeedWidget:label:
	 *
	 * The title of the feed.
	 */
    properties[PROP_TITLE] =
        g_param_spec_string ("title",
                             "Title",
                             "Feed title.",
                             NULL,
                             G_PARAM_READWRITE);

	/**
	 * GpodFeedWidget:image:
	 *
	 * The cover image of the feed.
	 */
    properties[PROP_IMAGE] =
        g_param_spec_string ("image",
                             "Image",
                             "Feed cover image.",
                             NULL,
                             G_PARAM_READWRITE);

    g_object_class_install_properties (object_class,
                                       NUM_PROPERTIES,
                                       properties);

    gtk_widget_class_set_template_from_resource (widget_class,
                                                 PACKAGE_RESOURCES "/ui/widgets/feed.ui");

    gtk_widget_class_bind_template_child_private (widget_class, GpodFeedWidget, box);
    gtk_widget_class_bind_template_child_private (widget_class, GpodFeedWidget, image);
    gtk_widget_class_bind_template_child_private (widget_class, GpodFeedWidget, title);
}

/**
 * gpod_feed_widget_new:
 *
 * Creates a new #GpodFeedWidget, a custom widget to show all feeds user is
 * subscribed to.
 *
 * ## Usage
 * |[
 * GpodFeedWidget *feed_widget;
 *
 * feed_widget = gpod_feed_widget_new ();
 * ]|
 *
 * Returns: #GpodFeedWidget
 */
GpodFeedWidget *
gpod_feed_widget_new ()
{
    return g_object_new (GPOD_TYPE_FEED_WIDGET,
                         NULL);
}

